==> join-0.4.tar.gz: join-0.4/server.c <==
/* server.c */
/* This is part of the source for the Core Join Calculus Compiler cjcc */
/* Copyright (C) 1996 Peter Selinger */
/* This is free software under the terms of the GNU General Public License. */

/*  ----------------------serverdef.c-------------------------------- */

void printvalue(FILE *file, value v)
{
  switch (v.type) {
  case INT:
    fprintf(file, "%d", v.c.i);
    break;
  case STR:
    fprintf(file, "%s", v.c.s);
    break;
  case BOOLE:
    fprintf(file, "%s", v.c.b ? "true" : "false");
    break;
  case CHANNEL:
    fprintf(file, 
	    "[channel %d (arity %d) of incarnation %d of definition %d]", 
	    v.c.chan.entry, v.c.chan.inc->arity[v.c.chan.entry], 
	    v.c.chan.inc, v.c.chan.inc->code);
    break;
  case WILDCARD:
    fprintf(file, "*");
    break;
  }
}

#define SACCSIZE 255

int readint(FILE *file, value *out_val)
{
  int i,stat;
  static char sacc[SACCSIZE];  /* string accumulator */
  
  stat=fscanf(file, "%s", sacc);
  if (stat==EOF)
    return EOF;
  if (stat==0)
    return 1;
  *out_val=inint(atoi(sacc));
  return 0;
}

/* Definition of the global server */

void server(incarnation *inc, int entry) 
{
  switch (entry) {
  case 0:      /* entry 0 is the 'write' server: write<printee,cont,exc> */
    if (queue_length(inc->q+entry) >= 3) {
      value printee,cont,exc;

      pop(&printee,inc->q+entry);
      pop(&cont,inc->q+entry);
      pop(&exc,inc->q+entry);
      printvalue(stdout,printee);
      sendalong0(cont);
    }
    break;
  case 1:      /* entry 1 is the 'readint' server: readint<cont,exc> */
    if (queue_length(inc->q+entry) >= 2) {
      value cont,exc;
      value result;
      int stat;
      pop(&cont,inc->q+entry);
      pop(&exc,inc->q+entry);
      stat=readint(stdin, &result);
      if (stat==EOF)
	sendalong1(exc,instr("End of file"));
      else if (stat)
	sendalong1(exc,instr("Bad input"));
      else 
	sendalong1(cont,result);
    }
    break;
                  /* Arithmetic operations */
  case 2:          /* entry 2 is test for equality */
  case 3:          /* entry 3 is 'plus': plus<first,second,cont,exc> */
  case 4:          /* entry 4 is 'minus' */
  case 5:          /* entry 5 is 'times' */
  case 6:          /* entry 6 is 'div' */
  case 7:          /* entry 7 is 'less' */
    if (queue_length(inc->q+entry) >= 4) {
      value first, second, cont, exc;
      pop(&first,inc->q+entry);
      pop(&second,inc->q+entry);
      pop(&cont,inc->q+entry);
      pop(&exc,inc->q+entry);
      if (first.type != INT || second.type !=INT) {
	if (entry==2)
	  sendalong1(exc,instr("Cannot test non-integers for equality"));
	else if (entry==7)
	  sendalong1(exc,instr("Cannot test non-integers for inequality"));
	else
	  sendalong1(exc,instr("Arithmetic operation on non-integers"));
	return;
      }
      switch (entry) {
      case 2: sendalong1(cont,inboole(first.c.i == second.c.i)); break;
      case 3: sendalong1(cont,inint(first.c.i + second.c.i)); break;
      case 4: sendalong1(cont,inint(first.c.i - second.c.i)); break;
      case 5: sendalong1(cont,inint(first.c.i * second.c.i)); break;
      case 6: 
	if (second.c.i==0)
	  sendalong1(exc,instr("Division by zero"));
	else
	  sendalong1(cont,inint(first.c.i / second.c.i)); 
	break;
      case 7: sendalong1(cont,inboole(first.c.i < second.c.i)); break;
      }
    }
    break;
  case 8:  /* entry 8 is if-then-else:  if<cond,thenk,elsek,exc> */
    if (queue_length(inc->q+entry) >= 4) {
      value cond, thenk, elsek, exc;
      pop(&cond,inc->q+entry);
      pop(&thenk,inc->q+entry);
      pop(&elsek,inc->q+entry);
      pop(&exc,inc->q+entry);
      if (cond.type != BOOLE) 
	sendalong1(exc,instr("First argument of 'if' is not a boolean"));
      else if (cond.c.b)
	sendalong0(thenk);
      else
	sendalong0(elsek);
    }
    break;
  }
} 


/*  -----------------------serverinc.c-----------------------------  */

/* create server incarnation */

{ incarnation *inc;
  value name_write, name_readint, name_equal;
  value name_plus, name_minus, name_times, name_div;
  value name_less, name_if;

  if ((inc=(incarnation *)calloc(1,sizeof(incarnation)))==NULL
      || (inc->q=(queue *)calloc(9,sizeof(queue)))==NULL
      || (inc->v=(value *)calloc(0,sizeof(value)))==NULL
      || (inc->arity=(int *)calloc(9,sizeof(int)))==NULL)
    run_time_err("Out of Memory");
  inc->code = server;
  { int i;
    for (i=0; i<9; i++)
      init_queue(&(inc->q[i]));
  }
  inc->arity[0]=3;
  inc->arity[1]=2;
  inc->arity[2]=4;
  inc->arity[3]=4;
  inc->arity[4]=4;
  inc->arity[5]=4;
  inc->arity[6]=4;
  inc->arity[7]=4;
  inc->arity[8]=4;
  name_write.type=CHANNEL;
  name_write.c.chan.inc=inc;
  name_write.c.chan.entry=0;
  name_readint.type=CHANNEL;
  name_readint.c.chan.inc=inc;
  name_readint.c.chan.entry=1;
  name_equal.type=CHANNEL;
  name_equal.c.chan.inc=inc;
  name_equal.c.chan.entry=2;
  name_plus.type=CHANNEL;
  name_plus.c.chan.inc=inc;
  name_plus.c.chan.entry=3;
  name_minus.type=CHANNEL;
  name_minus.c.chan.inc=inc;
  name_minus.c.chan.entry=4;
  name_times.type=CHANNEL;
  name_times.c.chan.inc=inc;
  name_times.c.chan.entry=5;
  name_div.type=CHANNEL;
  name_div.c.chan.inc=inc;
  name_div.c.chan.entry=6;
  name_less.type=CHANNEL;
  name_less.c.chan.inc=inc;
  name_less.c.chan.entry=7;
  name_if.type=CHANNEL;
  name_if.c.chan.inc=inc;
  name_if.c.chan.entry=8;

==> join-0.5.tar.gz: join-0.5/server.c <==
/* server.c */
/* This is part of the source for the Core Join Calculus Compiler cjcc */
/* Copyright (C) 1996 Peter Selinger */
/* This is free software under the terms of the GNU General Public License. */

/*  ----------------------serverdef.sc-------------------------------- */

void printvalue(FILE *file, value v)
{
  switch (v.type) {
  case INT:
    fprintf(file, "%d", v.c.i);
    break;
  case STR:
    fprintf(file, "%s", v.c.s);
    break;
  case BOOLE:
    fprintf(file, "%s", v.c.b ? "true" : "false");
    break;
  case CHANNEL:
    fprintf(file, 
	    "[channel %d (arity %d) of incarnation %d of definition %d]", 
	    v.c.chan.entry, v.c.chan.inc->arity[v.c.chan.entry], 
	    v.c.chan.inc, v.c.chan.inc->code);
    break;
  case WILDCARD:
    fprintf(file, "*");
    break;
  }
}

#define SACCSIZE 255

int readint(FILE *file, value *out_val)
{
  int i,stat;
  static char sacc[SACCSIZE];  /* string accumulator */
  
  stat=fscanf(file, "%s", sacc);
  if (stat==EOF)
    return EOF;
  if (stat==0)
    return 1;
  *out_val=inint(atoi(sacc));
  return 0;
}

/* Definition of the global server */

void server(incarnation *inc, int entry) 
{
  switch (entry) {
  case 0:      /* entry 0 is the 'write' server: write<printee,cont,exc> */
    if (queue_length(inc->q+entry) >= 3) {
      value printee,cont,exc;

      pop(&printee,inc->q+entry);
      pop(&cont,inc->q+entry);
      pop(&exc,inc->q+entry);
      printvalue(stdout,printee);
      sendalong0(cont);
    }
    break;
  case 1:      /* entry 1 is the 'readint' server: readint<cont,exc> */
    if (queue_length(inc->q+entry) >= 2) {
      value cont,exc;
      value result;
      int stat;
      pop(&cont,inc->q+entry);
      pop(&exc,inc->q+entry);
      stat=readint(stdin, &result);
      if (stat==EOF)
	sendalong1(exc,instr("End of file"));
      else if (stat)
	sendalong1(exc,instr("Bad input"));
      else 
	sendalong1(cont,result);
    }
    break;
                  /* Arithmetic operations */
  case 2:          /* entry 2 is test for equality */
  case 3:          /* entry 3 is 'plus': plus<first,second,cont,exc> */
  case 4:          /* entry 4 is 'minus' */
  case 5:          /* entry 5 is 'times' */
  case 6:          /* entry 6 is 'div' */
  case 7:          /* entry 7 is 'less' */
    if (queue_length(inc->q+entry) >= 4) {
      value first, second, cont, exc;
      pop(&first,inc->q+entry);
      pop(&second,inc->q+entry);
      pop(&cont,inc->q+entry);
      pop(&exc,inc->q+entry);
      if (first.type != INT || second.type !=INT) {
	if (entry==2)
	  sendalong1(exc,instr("Cannot test non-integers for equality"));
	else if (entry==7)
	  sendalong1(exc,instr("Cannot test non-integers for inequality"));
	else
	  sendalong1(exc,instr("Arithmetic operation on non-integers"));
	return;
      }
      switch (entry) {
      case 2: sendalong1(cont,inboole(first.c.i == second.c.i)); break;
      case 3: sendalong1(cont,inint(first.c.i + second.c.i)); break;
      case 4: sendalong1(cont,inint(first.c.i - second.c.i)); break;
      case 5: sendalong1(cont,inint(first.c.i * second.c.i)); break;
      case 6: 
	if (second.c.i==0)
	  sendalong1(exc,instr("Division by zero"));
	else
	  sendalong1(cont,inint(first.c.i / second.c.i)); 
	break;
      case 7: sendalong1(cont,inboole(first.c.i < second.c.i)); break;
      }
    }
    break;
  case 8:  /* entry 8 is if-then-else:  if<cond,thenk,elsek,exc> */
    if (queue_length(inc->q+entry) >= 4) {
      value cond, thenk, elsek, exc;
      pop(&cond,inc->q+entry);
      pop(&thenk,inc->q+entry);
      pop(&elsek,inc->q+entry);
      pop(&exc,inc->q+entry);
      if (cond.type != BOOLE) 
	sendalong1(exc,instr("First argument of 'if' is not a boolean"));
      else if (cond.c.b)
	sendalong0(thenk);
      else
	sendalong0(elsek);
    }
    break;
  }
} 


/*  -----------------------serverinc.sc-----------------------------  */

/* create server incarnation */

{ incarnation *inc;
  value name_write, name_readint, name_equal;
  value name_plus, name_minus, name_times, name_div;
  value name_less, name_if;

  if ((inc=(incarnation *)calloc(1,sizeof(incarnation)))==NULL
      || (inc->q=(queue *)calloc(9,sizeof(queue)))==NULL
      || (inc->v=(value *)calloc(0,sizeof(value)))==NULL
      || (inc->arity=(int *)calloc(9,sizeof(int)))==NULL)
    run_time_err("Out of Memory");
  inc->code = server;
  { int i;
    for (i=0; i<9; i++)
      init_queue(&(inc->q[i]));
  }
  inc->arity[0]=3;
  inc->arity[1]=2;
  inc->arity[2]=4;
  inc->arity[3]=4;
  inc->arity[4]=4;
  inc->arity[5]=4;
  inc->arity[6]=4;
  inc->arity[7]=4;
  inc->arity[8]=4;
  name_write.type=CHANNEL;
  name_write.c.chan.inc=inc;
  name_write.c.chan.entry=0;
  name_readint.type=CHANNEL;
  name_readint.c.chan.inc=inc;
  name_readint.c.chan.entry=1;
  name_equal.type=CHANNEL;
  name_equal.c.chan.inc=inc;
  name_equal.c.chan.entry=2;
  name_plus.type=CHANNEL;
  name_plus.c.chan.inc=inc;
  name_plus.c.chan.entry=3;
  name_minus.type=CHANNEL;
  name_minus.c.chan.inc=inc;
  name_minus.c.chan.entry=4;
  name_times.type=CHANNEL;
  name_times.c.chan.inc=inc;
  name_times.c.chan.entry=5;
  name_div.type=CHANNEL;
  name_div.c.chan.inc=inc;
  name_div.c.chan.entry=6;
  name_less.type=CHANNEL;
  name_less.c.chan.inc=inc;
  name_less.c.chan.entry=7;
  name_if.type=CHANNEL;
  name_if.c.chan.inc=inc;
  name_if.c.chan.entry=8;
